/*-*-*-*-*-*-*-*-*-*-* (C) 2000 STMicroelectronics *-*-*-*-*-*-*-*-*-*-*-*-*-*

PROJECT  : ST7265 Media Access Library
COMPILER : ST7 C

MODULE  :  MAL_All.c
VERSION :  V 0.5

CREATION DATE :  04/12/2000

AUTHOR : Liang Ping
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

DESCRIPTION : All functions except those with individual one

-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

Modification:
	16/05/2002 Patch for NAND Thumb drive

-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	VARIABLES	VARIABLES	VARIABLES	VARIABLES	VARIABLES	VARIABLES
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
#include "define.h"
#include "Map_7265.h"
#include "Mconfig.h"		// Media configuration
#include "MSL.h"
#include "MAL_Map.h"
#include "MAL.h"
#include "2K_Mass.h"
#include "DTC_Func.h"
#include "NAND.h"

extern void Deselect(void);

#define USE_CACHE0	0
//#define PAGE2K_BAD

/*-*-*-*-*-*-*-*-*-*-*-*-*-* Variable declaration *-*-*-*-*-*-*-*-*-*-*-*-*-*/
#pragma DATA_SEG SHORT MAL_RAM0	// Frequently used variable
// SMC_Wstate keeps track the state of the SMC writing procedure
// Bits definition:
//	bit0	= 0 actions on new cluster
//			= 1 actions on old cluster
//	bit1-0	= 00 MARKing
//			= 01 COPYing
//			= 10 WRITE to new cluster
//			= 11 WRITE to old cluster
//	bit3-2	= 00 PRE actions
//			= 01 POST actions
//			= 10 HOLE actions
//			= 11 HOLE or FULL actions
/*static*/ unsigned char	SMC_Wstate;		// the state of the SMC writing procedure
#define SMC_OLD_CLUSTER			0x01
#define	SMC_WRITE_FLAG			0x02
#define SMC_PRE_ACTION			0x00
#define SMC_POST_ACTION			0x04
#define SMC_HOLE_ACTION			0x08
#define SMC_FULL_ACTION			0x0C
#define SMC_ERROR_FLAG			0x10
#if ALLOW_OUT_RANGE
#define SMC_OUT_RANGE			(0x20 | SMC_WRITE_FLAG)
#endif

#define SMC_PREMARK			(SMC_PRE_ACTION | !SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// 0
#define SMC_PRECOPY			(SMC_PRE_ACTION | !SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// 1
#define SMC_PRENEWWRITE		(SMC_PRE_ACTION | SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// 2
#define SMC_PREOLDWRITE		(SMC_PRE_ACTION | SMC_WRITE_FLAG | SMC_OLD_CLUSTER)		// 3
#define SMC_POSTMARK		(SMC_POST_ACTION | !SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// 4
#define SMC_POSTCOPY		(SMC_POST_ACTION | !SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// 5
#define SMC_POSTNEWWRITE	(SMC_POST_ACTION | SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// 6
#define SMC_POSTOLDWRITE	(SMC_POST_ACTION | SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// 7
#define SMC_PREHOLEMARK		(SMC_HOLE_ACTION | !SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// 8
#define SMC_PREHOLECOPY		(SMC_HOLE_ACTION | !SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// 9
#define SMC_HOLENEWWRITE	(SMC_HOLE_ACTION | SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// A
#define SMC_HOLEOLDWRITE	(SMC_HOLE_ACTION | SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// B
#define SMC_POSTHOLEMARK	(SMC_FULL_ACTION | !SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// C
#define SMC_POSTHOLECOPY	(SMC_FULL_ACTION | !SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// D
#define SMC_FULLNEWWRITE	(SMC_FULL_ACTION | SMC_WRITE_FLAG | !SMC_OLD_CLUSTER)	// E
#define SMC_FULLOLDWRITE	(SMC_FULL_ACTION | SMC_WRITE_FLAG | SMC_OLD_CLUSTER)	// F

unsigned char SMC_Cluster_Size;				// Size of a cluster
											//statuc removed for 2K page
#define LOOKUP_THRESHOLD	250
unsigned char SubBlock;						//block with in a page for 256MB
static unsigned char Col;					//block with in a page for 256MB
unsigned char SMC_Zone;
static unsigned char SMC_Lookup_Zone0;		// Zone of table0
static unsigned char SMC_Lookup_Zone1;		// Zone of table1
static unsigned char SMC_Lookup_Count0;		// Visit count of table0
static unsigned char SMC_Lookup_Count1;		// Visit count of table1
static unsigned short *SMC_Lookup_pTable;	// The pointer of the current table
// When the zone 0 is not in one of two lookup table
// The follow small lookup table keeps the initial few items of zone 0
static unsigned char SMC_Lookup_nCache0;	// number of items in zone 0 Cache
											// number of sectors for MBR, PBR, FAT & Root
static unsigned char flag2k;

#ifdef THUMB_DRIVE
static unsigned char NAND_Chip;				// The current chip number
#endif

#if USE_CACHE0
#ifdef THUMB_DRIVE
#define CACHE0_NUM			16
#else
#define CACHE0_NUM			5
#endif
#define CACHE0_FREE			3
static unsigned short SMC_Lookup_pCache0[CACHE0_NUM];
// The following array is used to cache the free cluster
// for zone 0 when zone 0 is not in the lookup table area
static unsigned short SMC_Lookup_pFree0[CACHE0_FREE];
#endif // USE_CACHE0

static unsigned short SMC_Paddr;
static unsigned short SMC_Raddr;			// An old physical cluster
static unsigned int SMC_Npage;				// data type changed for 2K page.MAx page = 256
static unsigned int SMC_Npage512;			// variable added for 2K dec/inc blocks
unsigned char SMC_Ppage;

unsigned short SMC_Laddr;
unsigned long malPaddr;
unsigned long malRaddr;
//unsigned char malNpage;
unsigned int malNpage;					// data type changed for 2kflash
unsigned int Test_page;		// data type changed for 2kflash

#ifndef THUMB_DRIVE
static unsigned char SMC_4MB;				// The SMC is a 4MB card
#endif
static unsigned char DTC_Error;
static unsigned char DTC_Current_Func;
#define DTC_SMC_READ	0x01
#define DTC_SMC_WRITE	0x02
#define DTC_SMC_FORMAT	0x04

#pragma DATA_SEG MAL_RAM
#ifndef THUMB_DRIVE
static unsigned char SMC_Addr_Bytes;
static unsigned char CardID;				// added for 2K Flash
#endif

/*-*-*-*-*-*-*-*-*-*-*-*-*-* Variable declaration *-*-*-*-*-*-*-*-*-*-*-*-*-*/

// Buffer for redundant area
typedef struct _SPARE_AREA {
	union {
		unsigned long	dwReserved;
		unsigned short	wReserved[2];
	} R;
	unsigned char	Data_Status;
	unsigned char	Block_Status;
	unsigned short	Block_Address1;
	unsigned char	ECC2[3];
	unsigned short	Block_Address2;
	unsigned char	ECC1[3];
} SPARE_AREA;
static SPARE_AREA	SMC_Spare;

#define	Map_To_DTC_Addr(Target, zone, addr, page)	\
	{	((_UDWORD*)&Target)->byte[3] = LOBYTE((unsigned int)(addr)) | (unsigned char)(page);	\
		((_UDWORD*)&Target)->byte[2] = HIBYTE((unsigned int)(addr));	\
		if (SMC_Cluster_Size == 32) {	\
			if (zone & 1)	\
				((_UDWORD*)&Target)->byte[2] |= 0x80;	\
			((_UDWORD*)&Target)->byte[1] = (unsigned char)zone >> 1;	\
		}	\
		else	\
			((_UDWORD*)&Target)->byte[1] = 0;	\
		((_UDWORD*)&Target)->byte[0] = 0;		\
	} 

#define	Map_To_DTC_Addr256(Target, zone, addr, page, col) \
	{	if (SMC_Cluster_Size == 64) { \
			((_UDWORD*)&Target)->byte[3] = ((unsigned char)(col)); \
			((_UDWORD*)&Target)->byte[2] = LOBYTE((unsigned int)(addr)) | (unsigned char)(page); \
			((_UDWORD*)&Target)->byte[1] = HIBYTE((unsigned int)(addr)); \
			((_UDWORD*)&Target)->byte[0] = zone; \
		} \
		else { \
			((_UDWORD*)&Target)->byte[3] = LOBYTE((unsigned int)(addr)) | (unsigned char)(page);	\
			((_UDWORD*)&Target)->byte[2] = HIBYTE((unsigned int)(addr));	\
			if (SMC_Cluster_Size == 32) { \
				if (zone & 1) \
					((_UDWORD*)&Target)->byte[2] |= 0x80;	\
				((_UDWORD*)&Target)->byte[1] = (unsigned char)zone >> 1;	\
			}	\
			else	\
				((_UDWORD*)&Target)->byte[1] = 0;	\
			((_UDWORD*)&Target)->byte[0] = 0;		\
		}	\
	}
	

#define	Map_DTC_Addr(zone, addr, page)	\
	(SMC_Cluster_Size == 32) ?	\
		/* 11111111				ZZZ:		physical zone				*/ \
		/* 765432109876543210	BBBBBBBBBB: physical cluster address	*/ \
		/* ZZZBBBBBBBBBBPPPPP:	PPPPP:		physical page address		*/ \
		((DWORD)(zone) << 15) | (unsigned int)(addr) | (unsigned char)(page) :	\
		/* 1111														*/ \
		/* 32109876543210	BBBBBBBBBB: physical cluster address	*/ \
		/* BBBBBBBBBBPPPP:	PPPP:		physical page address		*/ \
		(unsigned int)(addr) | (unsigned char)(page);


#pragma CODE_SEG SMC_CODE

//*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
//	Functions for the NAND Thumb-Drive
//*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
#ifdef THUMB_DRIVE

// The value expresses number of zone's
// 1=16MB, 2=32MB, 4=64MB, 8=128MB, etc.
// 3=48MB, 5=80MB, 6=96MB, 7=112MB, etc.
static unsigned char Chip_Size[MAX_NUM_NAND];
static unsigned char Cluster_Size[MAX_NUM_NAND];

//#define gZONE(Chip, Zone)	(((Chip)<<4)|Zone)	// Global zone number
//for debug
#define gZONE(Chip, Zone)	(((Chip)<<5)|Zone)	// Global zone number

void NAND_Select(unsigned char iChip)
{
	switch (iChip) {
	case 0:	SELECT_NAND0; break;
	case 1:	SELECT_NAND1; break;
	case 2:	SELECT_NAND2; break;
	case 3:	SELECT_NAND3; break;
	default: DESELECT_NAND; return;
	}
	
	if ((Chip_Size[iChip] >= 4) && (Chip_Size[iChip]< 16))
		DTC_SMC_Address_bytes = 4;
	else if (Chip_Size[iChip] >= 16)
		DTC_SMC_Address_bytes = 5;	//if device is of 256MB
	else
		DTC_SMC_Address_bytes = 3;

	SMC_Cluster_Size = Cluster_Size[iChip];
}

unsigned char NAND_Capacity(unsigned char iChip)
{
	unsigned char CardID;
	unsigned char ChipSize, ClusterSize;

	NAND_Select(iChip);
	CardID = DTC_SMC_ReadID();
	Deselect();				// Deselect the NAND flash

	ClusterSize = 32;

	if (CardID == 0x73)			// 16MB
		ChipSize = 1;
	else if (CardID == 0x75)	// 32MB
		ChipSize = 2;
	else if (CardID == 0x76)	// 64MB
		ChipSize = 4;
	else if (CardID == 0x79)	// 128MB
		ChipSize = 8;
	else if	(CardID == 0xF1) {	// 128MB-2K
		ChipSize = 8;
		ClusterSize = 64;
	}
	else if(CardID == 0xda){	// 256MB-2K
		ChipSize = 16;
		ClusterSize = 64;
	}
	else if(CardID == 0xdc){	// 512MB-2K
		ChipSize = 32;
		ClusterSize = 64;
	}
	else
		ChipSize = 0;

	Chip_Size[iChip] = ChipSize;
	Cluster_Size[iChip] = ClusterSize;

	return ChipSize;
}

unsigned char NAND_Read_Capacity(void)
{
	unsigned char iChip;
	unsigned char TotalSize;

	TotalSize = 0;
	for (iChip = 0; iChip < MAX_NUM_NAND; iChip++)
		TotalSize += NAND_Capacity(iChip);

	if (TotalSize == 0 || Chip_Size[0] == 0)
		return MAL_CARD_UNKNOWN;

	asm {
		CLR		A
		LD		MAL_Capacity:3, A
		LD		MAL_Capacity, A
		LD		A, TotalSize
		LD		X, #125
		MUL		X, A
		LD		MAL_Capacity:1, X
		LD		MAL_Capacity:2, A
	}
//	MAL_Capacity = (unsigned long)TotalSize * 32000L;
	if (TotalSize <= (32 >> 4))			// <= 32MB
		SMC_Lookup_nCache0 = 0;			// No zone0 cache with card < 64MB
	else
		SMC_Lookup_nCache0 = 5;			// Use the maximum value

	return MAL_GOOD;
}

void Get_Chip_Number(void)
{
	unsigned char iChip;
	unsigned long Boundary;

	NAND_Chip = 0;
	for (iChip = 0; iChip < MAX_NUM_NAND; iChip++) {
		asm {
			CLR		A
			LD		Boundary:3, A
			LD		Boundary, A
			LD		X, iChip
			LD		A, (Chip_Size,X)
			LD		X, #125
			MUL		X, A
			LD		Boundary:1, X
			LD		Boundary:2, A
		}
		// Boundary = (unsigned long)Chip_Size[iChip] * 32000L;
		if (MAL_Block_Address < Boundary)
			break;

		MAL_Block_Address -= Boundary;
		NAND_Chip++;
	}
	SMC_Cluster_Size = Cluster_Size[NAND_Chip];
}

// Pre-declarations
void SMC_Copy_Lookup_Cache0();
void SMC_Convert_Lookup(unsigned short *LUT);
/*
void show_LED(void)
{
	PDDR |= 0x01;
	PDDR |= 0x04;
	if ((SMC_Lookup_Zone0 & 0xF0) == 0)
		PDDR &= ~0x01;
	if ((SMC_Lookup_Zone0 & 0xF0) == 0x10)
		PDDR &= ~0x04;
	if ((SMC_Lookup_Zone1 & 0xF0) == 0)
		PDDR &= ~0x01;
	if ((SMC_Lookup_Zone1 & 0xF0) == 0x10)
		PDDR &= ~0x04;
}
*/
void NAND_Lookup_Updates(unsigned char Zone)
{
	malRaddr = 1024;		// number of clusters in the zone
	malNpage = SMC_Cluster_Size;
	if (SMC_Cluster_Size == 64) {
		Buffer_Param[0x00] = 0x00;	//flag to differnetiate between 2K and 512
		Buffer_Param[0x02] = 0x00;	//1st cmd for 2k
		Buffer_Param[0x03] = 0xe3;	// adr for page byte
	}
	else {
		Buffer_Param[0x00] = 0x01;	//flag to differnetiate between 2K and 512
		Buffer_Param[0x02] = 0x50;	//1st cmd for 512
		Buffer_Param[0x03] = 0xe4;	// adr for page byte
	}
//	Map_To_DTC_Addr(malPaddr, Zone, 0, 0);	//commented for 2k
	Map_To_DTC_Addr256(malPaddr, Zone, 0, 0,2);	//new for 2K page 
	DTC_SMC_Map_Zone(SMC_Lookup_pTable);
	SMC_Convert_Lookup(SMC_Lookup_pTable);

	Deselect();				// Deselect the NAND flash
}

void NAND_Lookup_Update0(unsigned char Chip, unsigned char Zone)
{
	NAND_Select(Chip);

	SMC_Lookup_pTable = SMC_Lookup_Table0;
#if USE_CACHE0
	if (SMC_Lookup_Zone0 == 0)
		SMC_Copy_Lookup_Cache0();
#endif
	SMC_Lookup_Count0 = 0;
	SMC_Lookup_Zone0 = gZONE(Chip, Zone);

//show_LED();

	NAND_Lookup_Updates(Zone);
}

//#pragma NO_OVERLAP
void NAND_Lookup_Update1(unsigned char Chip, unsigned char Zone)
{
	NAND_Select(Chip);

	SMC_Lookup_pTable = SMC_Lookup_Table1;
#if USE_CACHE0
	if (SMC_Lookup_Zone1 == 0)
		SMC_Copy_Lookup_Cache0();
#endif
	SMC_Lookup_Count1 = 0;
	SMC_Lookup_Zone1 = gZONE(Chip, Zone);

//show_LED();

	NAND_Lookup_Updates(Zone);
}

static void NAND_INC_Logical_Addr(void)
{
	SMC_Laddr++;

/*----for 2K Flash-----	*/
	SubBlock = 0; //For next logical adr col offset is 0
	Buffer_Param[0x01] = 4; //No. of cols in Read operation is resetted
	Buffer_Param[0x25] = 0; //Col offset 0
	
	if (SMC_Laddr >= 1000) {
		SMC_Zone++;
		if (SMC_Cluster_Size != 64) {
			if (SMC_Zone == Chip_Size[NAND_Chip]) {
				goto common;
			//	SMC_Zone = 0;
			//	NAND_Chip++;
			//	while (Chip_Size[NAND_Chip] == 0)
			//		NAND_Chip++;		// Skip the bad of not exist chip
			//	NAND_Select(NAND_Chip);
			}
		}
		else {
			if (SMC_Zone == (Chip_Size[NAND_Chip] / 8)){
common:			SMC_Zone = 0;
				NAND_Chip++;
				while (Chip_Size[NAND_Chip] == 0)
					NAND_Chip++;
				NAND_Select(NAND_Chip);
			}
		}
		SMC_Laddr -= 1000;
	}
}

void Adjust_Laddr(void)
{
	while (SMC_Laddr >= 1000) {
		SMC_Zone++;
		SMC_Laddr -= 1000;
	}
}

void NAND_Get_Logical_Addr(void)
{
	unsigned short EndBlock;
	unsigned char SMC_Ppage512; //for claculating Endblock

	SMC_Zone = 0;
	SubBlock = 0; //initialise 
//	if (MAL_Block_Address == 0x2f)
//		asm nop;

	if (SMC_Cluster_Size == 32){
		asm {
			LD		A, MAL_Block_Address:3
			LD		X, MAL_Block_Address:2
			LD		Y, MAL_Block_Address:1
			SRL		Y
			RRC		X
			RRC		A	; 1st shift
			SRL		Y
			RRC		X
			RRC		A	; 2nd shift
			SRL		Y
			RRC		X
			RRC		A	; 3rd shift
			SRL		Y
			RRC		X
			RRC		A	; 4th shift
			SRL		Y
			RRC		X
			RRC		A	; 5th shift
			LD		SMC_Laddr:1, A
			LD		SMC_Laddr, X
		}	// The C line below costs too much
	
//	SMC_Laddr = (unsigned int)(MAL_Block_Address >> 5);

		Adjust_Laddr();
		SMC_Ppage = (unsigned char)MAL_Block_Address & 0x001F;
	}
	
	else { //for 2k Flash
		asm {
			LD		A, MAL_Block_Address:2
			LD		X, MAL_Block_Address:1
			LD		SMC_Laddr:1, A
			LD		SMC_Laddr, X
		}	// The C line below costs too much
	//	SMC_Laddr = (unsigned int)(MAL_Block_Address >> 8);
		Adjust_Laddr();
		SMC_Ppage512 = SMC_Ppage = (unsigned char)MAL_Block_Address & 0x00FF; //512 Byte page number for 2K
		SubBlock = (unsigned char) (SMC_Ppage % 4); //offset in the coloumn

		SMC_Ppage = SMC_Ppage / 4; //2K page number for 2K Flash
	}	


	if (SMC_Lookup_nCache0 == 0)	// Card < 64MB
		return;						// No cache management needed

	// Calculate the address of the last page: EndBlock 
	if (SMC_Cluster_Size != 64){
		asm {
			LD		A, MAL_Block_Numbers:1
			LD		X, MAL_Block_Numbers
			ADD		A, SMC_Ppage
			JRNC	sub_1
			INC		X
	sub_1:	SUB		A, #1
			JRNC	shift5
			DEC		X
	shift5:
			SRL		X
			RRC		A		; 1st shift
			SRL		X
			RRC		A		; 2nd shift
			SRL		X
			RRC		A		; 3rd shift
			SRL		X
			RRC		A		; 4th shift
			SRL		X
			RRC		A		; 5th shift
			ADD		A, SMC_Laddr:1
			LD		EndBlock:1, A
			LD		A, X
			ADC		A, SMC_Laddr
			LD		EndBlock, A
		}	// The following C line costs too much
	//	EndBlock = SMC_Laddr + ((SMC_Ppage + MAL_Block_Numbers - 1) >> 5);
	}
	else { //for 2K flash
		asm {
			LD		A, MAL_Block_Numbers:1
			LD		X, MAL_Block_Numbers
			ADD		A, SMC_Ppage
			JRNC	sub_2
			INC		X
	sub_2:	SUB		A, #1
			JRNC	shift8
			DEC		X
shift8:		LD		A, X
			ADD		A, SMC_Laddr:1
			LD		EndBlock:1, A
			CLR		A
			ADC		A, SMC_Laddr
			LD		EndBlock, A
		}	// The following C line costs a lot
	//	EndBlock = SMC_Laddr + ((SMC_Ppage512 + MAL_Block_Numbers - 1) >> 8);
	}


	if (EndBlock >= 1000) {		// The access will cross the zone boundary
		unsigned char ZoneNext;
		unsigned char ChipNext;

		ZoneNext = SMC_Zone + 1;
		ChipNext = NAND_Chip;
		if(SMC_Cluster_Size != 64) {
			if (ZoneNext >= Chip_Size[NAND_Chip]) {
				ZoneNext -= Chip_Size[NAND_Chip];
				goto com0;
		//		ChipNext++;
		//		while (Chip_Size[ChipNext] == 0)
		//			ChipNext++;			// Skip bad or not exist chips
			}
		}  
		else {
			if (ZoneNext >= (Chip_Size[NAND_Chip]/8)) {
				ZoneNext -= (Chip_Size[NAND_Chip]/8);
com0:			ChipNext++;
				while (Chip_Size[ChipNext] == 0)
					ChipNext++;			// Skip bad or not exist chips
			}
		}

		if (gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone0) { // current zone hits table0
			if (gZONE(ChipNext, ZoneNext) == SMC_Lookup_Zone1)
				return;
			
			NAND_Lookup_Update1(ChipNext, ZoneNext);		// update lookup tabke 1 for ZoneNext
		}
		else if (gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone1) {// current zone hits table1
			if (gZONE(ChipNext, ZoneNext) == SMC_Lookup_Zone0)
				return;

			NAND_Lookup_Update0(ChipNext, ZoneNext);		// update lookup table 0 for ZoneNext
		}
		else if (gZONE(ChipNext, ZoneNext) == SMC_Lookup_Zone0)	// next zone hits table0
			NAND_Lookup_Update1(NAND_Chip, SMC_Zone);		// update lookup table 1 for SMC_Zone
		else if (gZONE(ChipNext, ZoneNext) == SMC_Lookup_Zone1)	// next zone hits table1
			NAND_Lookup_Update0(NAND_Chip, SMC_Zone);		// updata lookup table 0 for SMC_Zone
		else {
			NAND_Lookup_Update0(NAND_Chip, SMC_Zone);		// updata lookup table 0 for SMC_Zone
			NAND_Lookup_Update1(ChipNext, ZoneNext);		// updata lookup table 1 for ZoneNext
		}
	}
	else {		// The access will not cross the zone boundary
		if (gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone0
				|| gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone1)
			return;
		
#if USE_CACHE0
		if (NAND_Chip == 0)		// Cache has to be for chip0 only
			if (SMC_Zone == 0 && SMC_Laddr < SMC_Lookup_nCache0) {	// Hit on cache of zone0
				if (EndBlock < SMC_Lookup_nCache0)
					return;
			}
#endif
		// Going to make the lookup table for the new zone
		if (SMC_Lookup_Count0 < SMC_Lookup_Count1)
			NAND_Lookup_Update0(NAND_Chip, SMC_Zone);
		else
			NAND_Lookup_Update1(NAND_Chip, SMC_Zone);
	}
}

#endif	// THUMB_DRIVE

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
				Organization of the Lookup Table

  There are two lookup tables, table0 & table1. Table0 always keep the
  information for Zone 0 and the Table1 keeps the information for the current
  working Zone.

  A lookup table is a buffer with 1024 elements. The index of this buffer is
  the logical address and the content of each element is the corresponding
  physical address.
  Assume LUT[x] is the value of xth element:
	LUT[x] < 1024	LUT[x] is the physical address of logicall address x
	LUT[x] >= 1024	(LUT[x] & 1024) is a free physical address and will be
					allocated to the logical address of x.
  NOTE: LUT[x] IS CONVERTED TO DTC ADDRESS FOR FASTING ACCESS
		LUT[x] >> 5 or LUT[x] >> 4 is the real physical cluster address

  Using the lookup table:
  The first 1000 elements of a lookup table are used for mapping the logical
  address to physical address. All the bad physical block addresses are saved
  at the end of this table. Elements start from index 1000 are available to
  use for new allocation.
	x < 1000		LUT[x] is used for mapping
	x >= 1024 - n	LUT[x] is the physical address of a bad physical cluster
					n is the number of bad blocks.
	x >= 1000 & x < 1024 -n
					LUT[x] is the free physical cluster that can be allocated
  When an existing physical cluster is over-written, the free cluster on the
  index 1000 is used. Then the old cluster is erased and put in the location
  of index 1000.

-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
//#pragma NO_ENTRY
void SMC_Convert_Lookup(unsigned short *pLUT)
{
	unsigned short	value;
	int iLUT;

	for (iLUT = 1023; iLUT >= 0; iLUT--) {
		value = *pLUT;
		if (SMC_Cluster_Size == 32)
			*pLUT = value << 5;
		else if (SMC_Cluster_Size == 64){
			if (*pLUT & 0x400) {	//to chk if it is a mapped cluster
				*pLUT = value << 6;
				*pLUT |= 0x0010;
			}
			else {
				*pLUT = value << 6;
			//	*pLUT &= 0xffef;	// last 6bits must be 0 after shift
			}
		}
		else {
			value &= ~0x0400;
			*pLUT = value << 4;
		}
		if (value & 0x2000)
			*pLUT |= 0x01;		// bad cluster
		pLUT++;
	}
}

void SMC_Paddr_To_DTC_Addr(void)
{
	Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Paddr, 0,Col);
}

unsigned short SMC_Get_Free()
{
#if USE_CACHE0
	if (SMC_Lookup_pTable == SMC_Lookup_pCache0)
		return SMC_Lookup_pFree0[0];
#endif

#ifndef THUMB_DRIVE
	if (SMC_4MB)
		return SMC_Lookup_pTable[500];
#endif
	return SMC_Lookup_pTable[1000];
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Replace SMC_Laddr with SMC_Paddr and set SMC_Raddr on logical address 1000
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
void SMC_Update_Lookup()
{
#if ALLOW_OUT_RANGE
	if (Mal_Addr_Out_Range & 0x01)
		return;
#endif
	SMC_Lookup_pTable[SMC_Laddr] = SMC_Paddr;

	if (SMC_Wstate & SMC_OLD_CLUSTER) {
	//	malPaddr = Map_DTC_Addr(SMC_Zone, SMC_Raddr, 0);
//		Map_To_DTC_Addr(malPaddr, SMC_Zone, SMC_Raddr, 0);
		Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Raddr, 0,0);//for 2K Flash
		if (DTC_SMC_Erase() != 0x01)		// Erase the dirty cluster of malPaddr
			asm NOP;

#if USE_CACHE0
		if (SMC_Lookup_pTable == SMC_Lookup_pCache0)
			SMC_Lookup_pFree0[0] = SMC_Raddr | 0x8000;
		else
#endif

#ifndef THUMB_DRIVE
		if (SMC_4MB)
			SMC_Lookup_pTable[500] = SMC_Raddr | 0x8000;
		else {
#endif		
			if (SMC_Cluster_Size != 64)
				SMC_Lookup_pTable[1000] = SMC_Raddr | 0x8000;
			else
				SMC_Lookup_pTable[1000] = SMC_Raddr | 0x0010;//different map bit for 2K flash
#ifndef THUMB_DRIVE
		}
#endif
	}
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Convert the SMC_Laddr to SMC_Paddr in SMC_Zone
	return TRUE if the logical Cluster of SMC_Laddr exists
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
#if USE_CACHE0
void SMC_Copy_Lookup_Cache0()
{
	unsigned short *pW;

	// Copy the first few lookup items of zone0
	// They are PBR, MBR, FAT & ROOT
	asm {
		LD		X, SMC_Lookup_nCache0
		SLL		X
		DEC		X
copy_cache:
		LD		A, ([SMC_Lookup_pTable.w],X)
		LD		(SMC_Lookup_pCache0,X), A
		DEC		X
		JRPL	copy_cache
	}
//	for (index = Lookup_nCache-1; index >= 0; index--)
//		SMC_Lookup_pCache0[index] = SMC_Lookup_pTable[index];

	// Copy the first few free lookup items of zone0
	asm {
		LD		A, SMC_Lookup_pTable:1
		ADD		A, #0xD0
		LD		pW:1, A
		LD		A, SMC_Lookup_pTable
		ADC		A, #0x07
		LD		pW, A
	}
//	pW = SMC_Lookup_pTable+1000;

	asm {
		LD		X, #CACHE0_FREE*2-1
copy_free0:
		LD		A, ([pW.w], X)
		LD		(SMC_Lookup_pFree0,X), A
		DEC		X
		JRPL	copy_free0
	}

//	for (index = CACHE0_FREE-1; index >= 0; index--)
//		SMC_Lookup_pFree0[index] = pW[index];
}
#endif // USE_CACHE0

#pragma DATA_SEG DEFAULT_RAM	// After here, all variable will be in non-Zero page

#ifndef THUMB_DRIVE
void SMC_Lookup_Update0(unsigned char Zone)
{
	SMC_Lookup_pTable = SMC_Lookup_Table0;
#if USE_CACHE0
	if (SMC_Lookup_Zone0 == 0)
		SMC_Copy_Lookup_Cache0();
#endif
	SMC_Lookup_Count0 = 0;
	SMC_Lookup_Zone0 = Zone;

	malRaddr = 1024;		// number of clusters in the zone
	if (SMC_4MB)
		malRaddr = 512;
	malNpage = SMC_Cluster_Size;
	if (SMC_Cluster_Size == 64) {
		Buffer_Param[0x00] = 0x00;		//flag for recognition
		Buffer_Param[0x02] = 0x00;		//first cmd for Mapzone
		Buffer_Param[0x03] = 0xe3;		//Add for mazone
	}
	else {
		Buffer_Param[0x00] = 0x01; 
		Buffer_Param[0x02] = 0x50;
		Buffer_Param[0x03] = 0xe4;
	}
//	Map_To_DTC_Addr(malPaddr, Zone, 0, 0);
	Map_To_DTC_Addr256(malPaddr, Zone, 0, 0, 2);	//for 2K Flash
	DTC_SMC_Map_Zone(SMC_Lookup_pTable);
	SMC_Convert_Lookup(SMC_Lookup_pTable);
}

//#pragma NO_OVERLAP
void SMC_Lookup_Update1(unsigned char Zone)
{
	SMC_Lookup_pTable = SMC_Lookup_Table1;
#if USE_CACHE0
	if (SMC_Lookup_Zone1 == 0)
		SMC_Copy_Lookup_Cache0();
#endif
	SMC_Lookup_Count1 = 0;
	SMC_Lookup_Zone1 = Zone;

	malRaddr = 1024;		// number of clusters in the zone
//	if (SMC_4MB)			// 4MB does not use LOOKUP Table 1
//		malRaddr = 512;
	malNpage = SMC_Cluster_Size; 
	if (SMC_Cluster_Size == 64) {		//For 256MB
		Buffer_Param[0x00] = 0x00;		//flag for recognition
		Buffer_Param[0x02] = 0x00;		//first cmd for Mapzone
		Buffer_Param[0x03] = 0xe3;		//Adr for mapzone
	}
	else {
		Buffer_Param[0x00] = 0x01;
		Buffer_Param[0x02] = 0x50;
		Buffer_Param[0x03] = 0xe4;
	}
//	Map_To_DTC_Addr(malPaddr, Zone, 0, 0);
	Map_To_DTC_Addr256(malPaddr, Zone, 0, 0, 2);		//for 2K Flash
	DTC_SMC_Map_Zone(SMC_Lookup_pTable);
	SMC_Convert_Lookup(SMC_Lookup_pTable);
}
#endif

static unsigned char SMC_Log2Phy()
{
#if ALLOW_OUT_RANGE
	if (Mal_Addr_Out_Range & 0x03)
		return FALSE;	// The address is out of range now
#endif

#ifdef THUMB_DRIVE
	if (gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone0) {		// Hit on table0
#else
	if (SMC_Zone == SMC_Lookup_Zone0) {		// Hit on table0
#endif
		SMC_Lookup_pTable = SMC_Lookup_Table0;
		if (SMC_Lookup_Count0 != LOOKUP_THRESHOLD)
			SMC_Lookup_Count0++;	// increase the visit counter of this table
		if (SMC_Lookup_Count1 != 0)
			SMC_Lookup_Count1--;	// decrease the visit counter of another table
	}
#ifdef THUMB_DRIVE
	else if (gZONE(NAND_Chip, SMC_Zone) == SMC_Lookup_Zone1) {// Hit on table1
#else
	else if (SMC_Zone == SMC_Lookup_Zone1) {	// Hit on table1
#endif
		SMC_Lookup_pTable = SMC_Lookup_Table1;
		if (SMC_Lookup_Count1 != LOOKUP_THRESHOLD)
			SMC_Lookup_Count1++;	// increase the visit counter of this table
		if (SMC_Lookup_Count0 != 0)
			SMC_Lookup_Count0--;	// decrease the visit counter of another table
	}
	else {
		// No hit on both table
#if USE_CACHE0
		// The _SMC_Get_Logical_Addr() gurantees it is on zone0 cache
		SMC_Lookup_pTable = SMC_Lookup_pCache0;
#else
//		asm nop;	// There must be a bug if we are here
#endif
	}

	if (SMC_Cluster_Size == 64){
		SMC_Paddr = SMC_Lookup_pTable[SMC_Laddr];
		return ((SMC_Paddr & 0x10) == 0);		//for 2K Flash
	}

	SMC_Paddr = SMC_Lookup_pTable[SMC_Laddr];
	return ((SMC_Paddr & 0x8000) == 0);

//	SMC_Paddr = SMC_Lookup_pTable[SMC_Laddr];
//	return ((SMC_Paddr & 0x8000) == 0);
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Mark and add SMC_Zone:SMC_Paddr to end of the lookup table
	SMC_Laddr is the logical cluster number.
	On return: SMC_Zone:SMC_Paddr will be the good cluster form SMC_Laddr
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
void SMC_Lookup_Map_Bad(void)
{
	unsigned int	temp;
	unsigned short*	pBad;

#if USE_CACHE0
	if (SMC_Lookup_pTable != SMC_Lookup_pCache0) {
		pBad = SMC_Lookup_pTable + 1023;		// the last item
		while (*pBad & 0x0F)
			pBad--;

		temp = *pBad;
		*pBad = SMC_Paddr | 0x0001;
	}
	else {
		// shift the free cluster array
		// SMC_Lookup_pFree0[0] = SMC_Lookup_pFree0[1];
		// SMC_Lookup_pFree0[1] = SMC_Lookup_pFree0[2];
		asm {
			CLR		X
shift_free0:
			LD		A, (SMC_Lookup_pFree0:2,X)
			LD		(SMC_Lookup_pFree0,X), A
			INC		X
			CP		X, #CACHE0_FREE*2-3
			JRNE	shift_free0
		}
		temp = SMC_Lookup_pFree0[0];
	}
#else
	pBad = SMC_Lookup_pTable + 1023;		// the last item
	while (*pBad & 0x01)
		pBad--;

	temp = *pBad;
	*pBad = SMC_Paddr | 0x0001;
#endif // USE_CACHE0

	// ASSUME THIS IS A GOOD ONE
	if (SMC_Cluster_Size != 64)
		SMC_Paddr = temp & 0x7FFF;			// re-allocate a good cluster
	else
		SMC_Paddr = temp & 0xffef;			//different mapping bit for 2K
}

#ifndef THUMB_DRIVE
unsigned char SMC_Read_Capacity(void)
{
	unsigned char	CardID;

	CardID = DTC_SMC_ReadID();

	SMC_4MB = 0;
	MAL_Capacity = 0;
	SMC_Lookup_nCache0 = 0;		// No zone0 cache with card < 64MB
	SMC_Cluster_Size = 16;
	SMC_Addr_Bytes = 3;
	if (CardID == 0xE3 || CardID == 0xE5) {	// 4MB
		SMC_4MB = 1;
		MAL_Capacity = 8000;
	}
	else if (CardID == 0xE6)		// 8MB
		MAL_Capacity = 16000;
	else {
		SMC_Cluster_Size <<= 1;
		if (CardID == 0x73) {		// 16MB
			MAL_Capacity = 32000;
		}
		else if (CardID == 0x75) {	// 32MB
			MAL_Capacity = 64000;
		}
		else if (CardID == 0x76) {	// 64MB
			MAL_Capacity = 128000;
			SMC_Addr_Bytes = 4;
			SMC_Lookup_nCache0 = 4;
		}
		else if (CardID == 0x79) {	// 128MB
			MAL_Capacity = 256000;
			SMC_Addr_Bytes = 4;
			SMC_Lookup_nCache0 = 5;
		}
		else if (CardID == 0xF1){	// 128MB-2K
			MAL_Capacity = 256000;
			SMC_Addr_Bytes = 4; 
			SMC_Cluster_Size <<= 1;
			SMC_Lookup_nCache0 = 5;
		}
		else if (CardID == 0xDA){	// 256MB-2K
			MAL_Capacity = 512000;
			SMC_Addr_Bytes = 5; 
			SMC_Cluster_Size <<= 1;
			SMC_Lookup_nCache0 = 5;
		}
		else
			return MAL_CARD_UNKNOWN;
	}

	DTC_SMC_Address_bytes = SMC_Addr_Bytes;
	return MAL_GOOD;
}
#endif

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
#ifndef THUMB_DRIVE
static void SMC_INC_Logical_Addr(void)
{
	SMC_Laddr++; 
/****for 2K Flash*****/	
	SubBlock = 0;
	Buffer_Param[0x01] = 4; 
	Buffer_Param[0x25]= 0;
	
	if (SMC_Laddr >= 1000) {
		SMC_Zone++;
		SMC_Laddr -= 1000;
	}
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
void SMC_Get_Logical_Addr(void)
{
	unsigned short EndBlock;

	SMC_Zone = 0;
	if (SMC_Cluster_Size == 32) {
		asm {
			LD		A, MAL_Block_Address:3
			LD		X, MAL_Block_Address:2
			LD		Y, MAL_Block_Address:1
			SRL		Y
			RRC		X
			RRC		A	; 1st shift
			SRL		Y
			RRC		X
			RRC		A	; 2nd shift
			SRL		Y
			RRC		X
			RRC		A	; 3rd shift
			SRL		Y
			RRC		X
			RRC		A	; 4th shift
			SRL		Y
			RRC		X
			RRC		A	; 5th shift
			LD		SMC_Laddr:1, A
			LD		SMC_Laddr, X
		}	// The C line below costs too much
	//	SMC_Laddr = (unsigned int)(MAL_Block_Address >> 5);

		Adjust_Laddr();
		SMC_Ppage = (unsigned char)MAL_Block_Address & 0x001F;
	} 
	else if (SMC_Cluster_Size == 64){ //for 256 MB
		asm {
			LD		A, MAL_Block_Address:2
			LD		X, MAL_Block_Address:1
			LD		SMC_Laddr:1, A
			LD		SMC_Laddr, X
		}	// The C line below costs too much
	//	SMC_Laddr = (unsigned int)(MAL_Block_Address >> 8);

		Adjust_Laddr();
		SMC_Ppage = (unsigned char)MAL_Block_Address & 0x00FF;	// No. of small pages
		SubBlock = (unsigned char) (SMC_Ppage % 4);				// Col. offset in a big page

		SMC_Ppage = SMC_Ppage / 4;//No. of 2K page
	}

	else {
		asm {
			LD		A, MAL_Block_Address:3
			LD		X, MAL_Block_Address:2
			LD		Y, MAL_Block_Address:1
			SRL		Y
			RRC		X
			RRC		A	; 1st shift
			SRL		Y
			RRC		X
			RRC		A	; 2nd shift
			SRL		Y
			RRC		X
			RRC		A	; 3rd shift
			SRL		Y
			RRC		X
			RRC		A	; 4th shift
			LD		SMC_Laddr:1, A
			LD		SMC_Laddr, X
		}	// The C line below costs too much
	//	SMC_Laddr = (unsigned int)(MAL_Block_Address >> 4);
		SMC_Ppage = (unsigned char)MAL_Block_Address & 0x000F;
	}

	if (SMC_Lookup_nCache0 == 0)	// Card < 64MB
		return;						// No cache management needed

	if (SMC_Cluster_Size != 64){
		asm {
			LD		A, MAL_Block_Numbers:1
			LD		X, MAL_Block_Numbers
			ADD		A, SMC_Ppage
			JRNC	sub_1
			INC		X
	sub_1:	SUB		A, #1
			JRNC	shift5
			DEC		X
	shift5:
			SRL		X
			RRC		A		; 1st shift
			SRL		X
			RRC		A		; 2nd shift
			SRL		X
			RRC		A		; 3rd shift
			SRL		X
			RRC		A		; 4th shift
			SRL		X
			RRC		A		; 5th shift
			ADD		A, SMC_Laddr:1
			LD		EndBlock:1, A
			LD		A, X
			ADC		A, SMC_Laddr
			LD		EndBlock, A
		}	// The following C line costs too much
	} 
	else {
		asm {
			LD		A, MAL_Block_Numbers:1
			LD		X, MAL_Block_Numbers
			ADD		A, SMC_Ppage
			JRNC	sub_2
			INC		X
	sub_2:	SUB		A, #1
			JRNC	shift8
			DEC		X
	shift8:	LD		A, X
			ADD		A, SMC_Laddr:1
			LD		EndBlock:1, A
			CLR		A
			ADC		A, SMC_Laddr
			LD		EndBlock, A
		}	// The following C line costs too much
	}
//	EndBlock = SMC_Laddr + ((SMC_Ppage + MAL_Block_Numbers - 1) >> 5);

	if (EndBlock >= 1000) {		// The access will cross the zone boundary
		unsigned char ZoneNext;

		ZoneNext = SMC_Zone + 1;
		if (SMC_Zone == SMC_Lookup_Zone0) {		// current zone hits table0
			if (ZoneNext == SMC_Lookup_Zone1)
				return;

			SMC_Lookup_Update1(ZoneNext);		// update lookup tabke 1 for ZoneNext
		}
		else if (SMC_Zone == SMC_Lookup_Zone1) {// current zone hits table1
			if (ZoneNext == SMC_Lookup_Zone0)
				return;

			SMC_Lookup_Update0(ZoneNext);		// update lookup table 0 for ZoneNext
		}
		else if (ZoneNext == SMC_Lookup_Zone0)	// next zone hits table0
			SMC_Lookup_Update1(SMC_Zone);		// update lookup table 1 for SMC_Zone
		else if (ZoneNext == SMC_Lookup_Zone1)	// next zone hits table1
			SMC_Lookup_Update0(SMC_Zone);		// updata lookup table 0 for SMC_Zone
		else {
			SMC_Lookup_Update0(SMC_Zone);		// updata lookup table 0 for SMC_Zone
			SMC_Lookup_Update1(ZoneNext);		// updata lookup table 1 for ZoneNext
		}
	}
	else {		// The access will not cross the zone boundary
		if (SMC_Zone == SMC_Lookup_Zone0 || SMC_Zone == SMC_Lookup_Zone1)
			return;

#if USE_CACHE0
		if (SMC_Zone == 0 && SMC_Laddr < SMC_Lookup_nCache0) {	// Hit on cache of zone0
			if (EndBlock < SMC_Lookup_nCache0)
				return;
		}
#endif
		// Going to make the lookup table for the new zone
		if (SMC_Lookup_Count0 < SMC_Lookup_Count1)
			SMC_Lookup_Update0(SMC_Zone);
		else
			SMC_Lookup_Update1(SMC_Zone);
	}
}
#endif	// not THUMB_DRIVE

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Launch the DTC again for reading
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
void SMC_Read_Interrupt()
{
	_MAL_INC_Block_Param(SMC_Npage);	
//	MAL_Block_Numbers -= SMC_Npage;
//	MAL_Block_Address += SMC_Npage;
	if (DTC_Error) {
		_MAL_DEC_Block_Param(malNpage);
	//	MAL_Block_Numbers += malNpage;
	//	MAL_Block_Address -= malNpage;

		MAL_Error = MAL_RD_ERR;
		MAL_Finish(FALSE);

		DTC_Current_Func = 0;
		return;
	}
	if (MAL_Block_Numbers == 0) {
		MAL_Error = MAL_GOOD;
		MAL_Finish(TRUE);

		DTC_Current_Func = 0;
		return;
	}

	// If the previous DTC was reading odd number of pages
	// Clear the buffer manager flags and start from buffer0
	if (SMC_Npage & 1) {
		while (BUFSR & 0x06);		// Wait both buffer are empty
		BUFSR = 0x01;				// Clear Buffer status
		BUFSR = 0;
	}

	// Launch the next burst reading
#ifdef THUMB_DRIVE
	NAND_INC_Logical_Addr();
#else
	SMC_INC_Logical_Addr();
#endif

	if (SMC_Cluster_Size != 64){
		if (SMC_Cluster_Size > MAL_Block_Numbers)
			malNpage = (unsigned char)MAL_Block_Numbers;
		else
			malNpage = SMC_Cluster_Size;
	}
	else		// SMC_Cluster_Size = 64	// for 2k Flash
		if (MAL_Block_Numbers < 0x100)		// No. of small pages = 4 * big pages
			malNpage = (unsigned char)MAL_Block_Numbers;		
		else
			malNpage = 0x100;

	_MAL_BufMgr_Upload();		// Switch to UPLOAD mode

	SMC_Npage = malNpage;
	DTC_Current_Func = DTC_SMC_READ;
	if (SMC_Log2Phy() == 0) {
// WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
		// When the specified location does not exist
		// Have to return all 0xFF in the data block.
		DTC_Read_FF();
		return;
	}

	if (SMC_Paddr & 0x0F) {			// This is a bad cluster
		MAL_Error = MAL_BAD_MEDIUM;	// Too many bad clusters
		MAL_Finish(FALSE);

		DTC_Current_Func = 0;
		return;
	}

//	malPaddr = Map_DTC_Addr(SMC_Zone, SMC_Paddr, SMC_Npage);
//	Map_To_DTC_Addr(malPaddr, SMC_Zone, SMC_Paddr, 0);
	Col = 0;

	SMC_Paddr_To_DTC_Addr();
	DTC_SMC_Read_USB( /* malPaddr, malNpage */ );
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	MAL_SMC_Read
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
unsigned char SMC_Read(void)
{
#ifdef THUMB_DRIVE
	Get_Chip_Number();		// Map the MAL address within a chip

// Get the logical cluster address
	NAND_Get_Logical_Addr();		// SMC_Zone:SMC_Laddr:SMC_Ppage = MAL_Block_Address

	NAND_Select(NAND_Chip);			// Enable the chip select of the chip
#else
	DTC_SMC_Address_bytes = SMC_Addr_Bytes;
// Get the logical cluster address
	SMC_Get_Logical_Addr();		// SMC_Zone:SMC_Laddr:SMC_Ppage = MAL_Block_Address
#endif

	if (SMC_Cluster_Size != 64){
		Buffer_Param[0x00] = 0x01;	//flag to recognize 2K or 512 flash
		Buffer_Param[0x25] = 0x00; //1st adr byte to be sent 
	}
	else {
		asm {
			LD		A, SubBlock
			SWAP	A
			LD		Buffer_Param[0x25], A
		}
	//	if (SubBlock == 0) //1st col byte adr for 2K page
	//		Buffer_Param[0x25] = 0x00;
	//	else if (SubBlock == 1)
	//		Buffer_Param[0x25] = 0x10;
	//	else if (SubBlock == 2)
	//		Buffer_Param[0x25] = 0x20;
	//	else
	//		Buffer_Param[0x25] = 0x30;
		Buffer_Param[0x00] = 0x00;			//flag to recognize 2K or 512 flash 
		Buffer_Param[0x01] = (4 - SubBlock);//Number of partial pages to be read
	}

	MAL_Block_Finish = 0;

	if (SMC_Cluster_Size != 64)
		malNpage = SMC_Cluster_Size - SMC_Ppage;
	else {
		asm {
			CLR		malNpage
			LD		A, SMC_Ppage
			SLL		A
			SLL		A
			ADD		A, SubBlock
			NEG		A
			LD		malNpage:1, A
			JRNE	got_Npage
			INC		malNpage
got_Npage:
		}
	//	malNpage = SMC_Ppage * 4 + SubBlock;
	//	malNpage = 0x100 - malNpage;
	}

	if (malNpage > MAL_Block_Numbers)
		malNpage = (unsigned char)MAL_Block_Numbers;//No. of small pages to be read
	
	SMC_Npage = malNpage;
	DTC_Current_Func = DTC_SMC_READ;

	_MAL_BufMgr_Upload();		// Switch to UPLOAD mode

// Get the physical cluster address
	if (SMC_Log2Phy() == 0) {
		// When the specified location does not exist
		// Have to return all 0xFF in the data block.
		DTC_Read_FF();
		return MAL_GOOD;
	}

	if (SMC_Paddr & 0x0F) {			// This is a bad cluster
		MAL_Error = MAL_BAD_MEDIUM;	// Too many bad clusters
		MAL_Finish(FALSE);

		DTC_Current_Func = 0;
		return MAL_Error;
	}

//	Launch the DTC to copy the first data cluster
//	malPaddr = Map_DTC_Addr(SMC_Zone, SMC_Paddr, SMC_Ppage);

	Col = SubBlock * 2; //To get 2nd col. byte

	Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Paddr, SMC_Ppage,Col);
	
//	Map_To_DTC_Addr(malPaddr, SMC_Zone, SMC_Paddr, SMC_Ppage);
	DTC_SMC_Read_USB( /* malPaddr, malNpage */ );

	return MAL_GOOD;
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	MAL_Write();	MAL_Write();	MAL_Write();	MAL_Write();
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Launch the DTC again for writing
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
unsigned char SMC_Write(void);
unsigned short MAL_Block_Numbers_t;
#define PAGE2K_BAD
void SMC_Write_Interrupt()
{
Label_SMC_Write_Interrupt:
			// Some DTC routine is expected to finish with interrupt,
			// but it returns when it finishes.
			// This label is used to simulate the re-enter as interrupt.

#ifdef PAGE2K_BAD
if(SMC_Cluster_Size ==  64){
	if (DTC_Error) {
		if((SMC_Wstate != SMC_PREMARK) && (SMC_Wstate != SMC_PRECOPY)){
			malPaddr = (malPaddr & ~(0x3fff));
			DTC_SMC_Mark_Bad();
			SMC_Lookup_Map_Bad();
			if (SMC_Wstate & SMC_OLD_CLUSTER)
				SMC_Lookup_pTable[1000] = SMC_Paddr | 0x10;
			else
				SMC_Lookup_pTable[SMC_Laddr] = SMC_Paddr | 0x10;

			MAL_Error = MAL_WR_ERR;
			DTC_Error = 0; 
			MAL_Block_Numbers = MAL_Block_Numbers_t;	//ASk for all data in case of failure
			MAL_Finish(FALSE);
			return;
		}
		else{
			malPaddr = (malPaddr & ~(0x3fff));
			DTC_SMC_Mark_Bad();
			SMC_Lookup_Map_Bad();
			if (SMC_Wstate & SMC_OLD_CLUSTER) {
				SMC_Lookup_pTable[1000] = SMC_Paddr | 0x10;
			}
			else {
				SMC_Lookup_pTable[SMC_Laddr] = SMC_Paddr | 0x10;
			} 
			DTC_Error = 0;
			SMC_Write();
			return;
		}
	}
}
#endif

	if (SMC_Wstate & SMC_WRITE_FLAG) {
		if (SMC_Cluster_Size != 64) {
			if(DTC_Error)
				SMC_Npage -= malNpage;
			_MAL_INC_Block_Param(SMC_Npage);
		}
		else {
			if (DTC_Error)			//for 2K page Bad Block
				SMC_Npage512 -= malNpage;//No. of 512 byte pages decremented

			_MAL_INC_Block_Param(SMC_Npage512); 
		}
	}

	DTC_Current_Func = DTC_SMC_WRITE;

	switch (SMC_Wstate) {
	case SMC_PREMARK:		// 0
	case SMC_PRECOPY:		// 1
	case SMC_PREHOLEMARK:	// 8 Never use
	case SMC_PREHOLECOPY:	// 9 Never use
		malNpage = SMC_Cluster_Size - SMC_Npage;
		if (SMC_Cluster_Size != 64){
			if (malNpage > MAL_Block_Numbers) {
				malNpage = (unsigned char)MAL_Block_Numbers;
				SMC_Wstate |= SMC_HOLE_ACTION;
			}
			SMC_Npage = malNpage;	//counter for no. of pages to be written
		}
		else {
			malNpage = ((malNpage * 4) - SubBlock);
			if (malNpage > MAL_Block_Numbers) {
				malNpage = (unsigned char)MAL_Block_Numbers;
				SMC_Wstate |= SMC_HOLE_ACTION;
			//	SMC_Npage = malNPage;
			}
			SMC_Npage512 = malNpage; //for decrement of MAL_Block_Numbers, counter for 512 byte page
			SMC_Npage = (SubBlock + malNpage)/4;  //counter for no. of 2K pages
			if (SMC_Npage == 0 && SubBlock)
				SMC_Npage = 1;
		}
		Test_page = Test_page - malNpage;
		// Launch DTC to write several pages;
		SMC_Wstate |= SMC_WRITE_FLAG;
		_MAL_BufMgr_Download();
		DTC_SMC_Write_USB(0);
		// Now SMC_Wstate = SMC_PRENEWWRITE or SMC_PREOLDWRITE or
		//     SMC_Wstate = SMC_HOLENEWWRITE or SMC_HOLEOLDWRITE
		// SMC_Wstate = 2 : 3 : A : B
		return;

	case SMC_POSTNEWWRITE:	// 6
	case SMC_POSTOLDWRITE:	// 7
		// All data from USB has been written. MAL_Block_Numbers must be 0
		malNpage = (unsigned char)(SMC_Cluster_Size - SMC_Npage);
		SMC_Ppage = SMC_Npage;
		SMC_Wstate &= ~SMC_WRITE_FLAG;		// Clear WRITE flag

Post_Mark_Copy:
		if(SMC_Cluster_Size == 64){
			Col = ((Buffer_Param[0x24] + 0x02) & 0x06);  //get the number of coloumns after write operation

//			malNpage = malNpage - 1;

			if (Col == 0x02)		//set the 1st adr byte for src and destination
				SubBlock = 3;
			else if (Col == 0x04)
				SubBlock = 2;
			else if (Col == 0x06)
				SubBlock = 1;
			else {
				SubBlock = 0;
//				malNpage = malNpage + 1;		// One more big page to copy
			}
		}	// if it is 2K pag

		SMC_Npage = malNpage;
		if (SMC_Wstate & SMC_OLD_CLUSTER) {
			// This was an overwrite operation, copy the remain pages
			//	malRaddr = Map_DTC_Addr(SMC_Zone, SMC_Raddr, SMC_Ppage);
			//			Map_To_DTC_Addr(malRaddr, SMC_Zone, SMC_Raddr, SMC_Ppage);
			if (SMC_Ppage == 0x40 && malNpage == 0) {
				asm	NOP;
				goto Label_SMC_Write_Interrupt;
			}

			SMC_Npage = malNpage;
			Map_To_DTC_Addr256(malRaddr, SMC_Zone, SMC_Raddr, SMC_Ppage,0);//for 2K page
			Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Paddr, SMC_Ppage,0);//for 2K page
			
			if (SMC_Cluster_Size == 64){
				if ((Buffer_Param[0x24]/2+1) > SMC_Npage512) {	// The current big page is complete
					Buffer_Param[0x1A] = 0;		// No pre-copy nor post-copy
					if (SubBlock) {
						SubBlock = 0;
//						malNpage = malNpage + 1;		// One more big page to copy
					}
					else
						asm NOP;
				}
				else {	// Check if the current big page has small pages to copy
					if (SubBlock == 0)
						asm	NOP;
						
					SubBlock = Buffer_Param[0x24]/2 + 1;	// Remain small pages to copy
					Buffer_Param[0x18] = 0;			// Column start addrsss
					Buffer_Param[0x19] = 0;			// Must be 0x0000
					Buffer_Param[0x16] = SubBlock * 2;	// Length of blank area
					Buffer_Param[0x17] = SubBlock * 0x10;	// 0x210, 0x420, 0x630
					Buffer_Param[0x1A] = 3;
					if (malNpage == 0xffff)
						asm	NOP;
//					malNpage = malNpage + 1;		// One more big page to copy
				}
				((unsigned char *)(&(malPaddr)))[3] = 0x00; //col byte is always 0
//				Buffer_Param[0x27] = Buffer_Param[0x25]= 0x00; //col byte is 0 in copypages
			}
			
			// Launch the DTC to copy several pages
			// Copy malNpage pages from malRaddr to malPaddr
			_MAL_BufMgr_Normal();
			DTC_Error = DTC_SMC_Copy_Pages(); 
			DTC_Error--;
			// Now SMC_Wstate = SMC_POSTCOPY; // 5
		}
		else {
			// This was a new cluster operation, mark the remain pages
			// Launch the DTC to several several pages start from malPaddr
//----------------------------------------------------------------
			Buffer_Param[0x25] = ((4 - SubBlock) * 0x10) & 0x30;	// added by HYJ
			malNpage = Test_page / 4;
			SubBlock = Test_page % 4;
//			if (Buffer_Param[0x24] == 0 || SMC_Npage512 > 4 || (Buffer_Param[0x24] + SMC_Npage512) == 4)
//				malNpage = malNpage - 1;			// One more big page to Mark
//			else
//				asm	NOP;

			SMC_Npage = malNpage;
//----------------------------------------------------------------
			DTC_Error = DTC_SMC_Mark_Pages();
			DTC_Error--;
			// Now SMC_Wstate = SMC_POSTMARK;	// 4
		}
		// Mask off this line if DTC interrupts
		goto Label_SMC_Write_Interrupt;
		return;

	case SMC_HOLENEWWRITE:	// A
	case SMC_HOLEOLDWRITE:	// B
		SMC_Ppage += SMC_Npage;
		SMC_Wstate = (SMC_Wstate & SMC_OLD_CLUSTER) | SMC_POST_ACTION;	// Keep OLD flag
		malNpage = SMC_Cluster_Size - SMC_Ppage;	// Pages left to be marked or copied
		goto Post_Mark_Copy;	// The following actions are same as above

	case SMC_POSTMARK:		// 4
	case SMC_POSTCOPY:		// 5
	case SMC_POSTHOLEMARK:	// C
	case SMC_POSTHOLECOPY:	// D
	case SMC_PRENEWWRITE:	// 2
	case SMC_PREOLDWRITE:	// 3
	case SMC_FULLNEWWRITE:	// E
	case SMC_FULLOLDWRITE:	// F
		// Now we have finished one cluster
		SMC_Update_Lookup();

		if (MAL_Block_Numbers == 0) {
			// All data from the USB has been written to the SMC
			MAL_Error = MAL_GOOD;
			MAL_Finish(TRUE); 
			DTC_Current_Func = 0;
			return;
		}

#if ALLOW_OUT_RANGE
		if (Mal_Addr_Out_Range & 0x02) {
			SMC_Wstate = SMC_OUT_RANGE;
			goto Write_Out_Range;
		}
#endif
		// Further more data to be written to the media
#ifdef THUMB_DRIVE
		NAND_INC_Logical_Addr();
#endif

		SMC_Wstate = SMC_FULL_ACTION | SMC_WRITE_FLAG;
		if (SMC_Log2Phy()) {
			// This is an existing cluster
			SMC_Wstate |= SMC_OLD_CLUSTER;
			SMC_Raddr = SMC_Paddr;			// old cluster
			SMC_Paddr = SMC_Get_Free();		// new cluster
		}
		if (SMC_Cluster_Size == 64)
			SMC_Paddr &= 0xffef; //reset the flag for mapped 
		else
			SMC_Paddr &= 0x7FFF;

		SMC_Ppage = 0;

		// the next writing must start from beginning of the cluster
		malNpage = SMC_Cluster_Size;
		if (SMC_Cluster_Size != 64) {
			if (malNpage > MAL_Block_Numbers) {
				SMC_Wstate &= ~0x08;		// change from SMC_FULL_ACTION to SMC_POST_ACTION
				malNpage = (unsigned char)MAL_Block_Numbers;
			}
			//	malPaddr = Map_DTC_Addr(SMC_Zone, SMC_Paddr, 0);
			SMC_Npage = malNpage;
		}
		else {
			malNpage = malNpage * 4; //No. of 2K pages
			if (malNpage > MAL_Block_Numbers){
				SMC_Wstate &= ~0x08;		// change from SMC_FULL_ACTION to SMC_POST_ACTION
				malNpage = (unsigned char)MAL_Block_Numbers;
			}
			SMC_Npage512 = malNpage;		//for decrement of MAL_Block_Numbers   
			SMC_Npage = malNpage / 4;		//counter for 2K pages 
		}
		Test_page = 256 - malNpage;
	//	Map_To_DTC_Addr(malPaddr, SMC_Zone, SMC_Paddr, 0);
		Col = 0x00;
		Buffer_Param[0x25] = 0x00;			//added as it is the start of the cluster

		SMC_Paddr_To_DTC_Addr();

		// Launch DTC to write malPpage pages to malPaddr
		_MAL_BufMgr_Download();
		DTC_SMC_Write_USB(MAL_Block_Finish & 1);

		// Now SMC_Wstate = SMC_POSTNEWWRITE or SMC_POSTOLDWRITE or
		//	   SMC_Wstate = SMC_FULLNEWWRITE or SMC_FULLOLDWRITE
		// SMC_Wstate = 6 : 7 : E : F
		return;

#if ALLOW_OUT_RANGE
	case SMC_OUT_RANGE:
		if (MAL_Block_Numbers == 0) {
			// All data from the USB has been written to the SMC
			MAL_Error = MAL_GOOD;
			MAL_Finish(TRUE);

			DTC_Current_Func = 0;
			return;
		}

Write_Out_Range:
	// assume MAL_Block_Numbers < 256
	//	malNpage = SMC_Cluster_Size;
	//	if (malNpage > MAL_Block_Numbers)
	//		malNpage = (unsigned char)MAL_Block_Numbers;
		SMC_Npage = malNpage = (unsigned char)MAL_Block_Numbers;
		DTC_SMC_Dummy_Write(MAL_Block_Finish & 1);
		return;
#endif
	}
}

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	MAL_SMC_Write
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
static unsigned char l= 0;
unsigned char SMC_Write(void)
{
	Get_Chip_Number();					// Map the MAL address within a chip
	// Get the logical cluster address
	NAND_Get_Logical_Addr();			// SMC_Zone:SMC_Laddr:SMC_Ppage = MAL_Block_Address
	NAND_Select(NAND_Chip);				// Enable the chip select of the chip

	MAL_Block_Numbers_t = MAL_Block_Numbers;
	MAL_Block_Finish = 0;

	//set 1st col adr byte as per the adr issued by the PC
	asm {
		LD		A, SubBlock
		SWAP	A
		LD		Buffer_Param[0x25], A
	}
	Col = SubBlock * 2;					// set 2nd col adr byte

	// Get the physical cluster address
	SMC_Wstate = 0;							// PRE actions, NEW cluster, no WRITE
	if (SMC_Log2Phy()) {					// Check if it is an existing cluster
		SMC_Wstate |= SMC_OLD_CLUSTER;		// Yes, PRECOPY may be necessary
		SMC_Raddr = SMC_Paddr;				// old cluster
		SMC_Paddr = SMC_Get_Free();			// new cluster
	}
	if (SMC_Cluster_Size == 64)
		SMC_Paddr &= 0xffef;				// Reset the flag for mapped 
	else
		SMC_Paddr &= 0x7FFF;

	Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Paddr, 0, 0); //for 2K flash
	DTC_Current_Func = DTC_SMC_WRITE;

	if ((SMC_Ppage) || (SubBlock)) {	//for 2K flash
		// the new pages are not start from beginning of the cluster
//		Buffer_Param[0x25]= Buffer_Param[0x27] = 0x00;	// ALways from the start of the page start copy or mark
		SMC_Npage = malNpage = SMC_Ppage;
		_MAL_BufMgr_Normal();							// switch back to normal mode first
		if (SMC_Wstate & SMC_OLD_CLUSTER) {
			unsigned char Num_subBlock;

			Map_To_DTC_Addr256(malRaddr, SMC_Zone, SMC_Raddr, 0, 0);	//for 2K Flash

			if (SubBlock) {
				malNpage++;
				if (SMC_Ppage == 0)						// only copy <4 small pages
					Buffer_Param[0x1A] = 0x03;			// Flag for Pre-copy
				else
					Buffer_Param[0x1A] = 0x02;			// Flag for Post-copy
			}
			else
				Buffer_Param[0x1A] = 0x0;

			Buffer_Param[0x18] = SubBlock * 2;			// Column start addrsss
			Buffer_Param[0x19] = SubBlock * 0x10;		// 0x210, 0x420, 0x630
			
			Num_subBlock = 4 - SubBlock;
			if (Num_subBlock > MAL_Block_Numbers)
				Num_subBlock = MAL_Block_Numbers;
			Buffer_Param[0x16] = Num_subBlock * 2;		// Length of blank area
			Buffer_Param[0x17] = Num_subBlock * 0x10;	// 0x210, 0x420, 0x630
			
			// Launch the DTC to copy several pages
			// Copy malNpage pages from malRaddr to malPaddr
			DTC_Error = DTC_SMC_Copy_Pages();
			DTC_Error--;

			if (SMC_Cluster_Size == 64) {
				if (SubBlock) {
					Map_To_DTC_Addr256(malRaddr, SMC_Zone, SMC_Raddr, SMC_Ppage, 0); //for 2K Flash
					Map_To_DTC_Addr256(malPaddr, SMC_Zone, SMC_Paddr, SMC_Ppage, 0); //for 2K Flash
				}
//				if (SMC_Ppage != 0)
					((unsigned char *)(&(malPaddr)))[3] = Col;
				Buffer_Param[0x25] = Col << 3;
			}
			// Now SMC_Wstate == SMC_PRECOPY
		}
		else {	// This is a new logical cluster
			// Launch the DTC to mark several pages
			Buffer_Param[0x25] = 0;	// Added by HYJ

			Test_page = 256 - (malNpage * 4) - SubBlock;
			
			DTC_Error = DTC_SMC_Mark_Pages();				// Mark the starting pages
			DTC_Error--;
			
			if (SMC_Cluster_Size == 64) {
				Buffer_Param[0x26] = Buffer_Param[0x12];	//as 0xe6 may contain 2 after markpages

				asm {
					LD		A, SubBlock
					SLL		A
					LD		malPaddr:3, A
					LD		A, SubBlock
					SWAP	A
					LD		Buffer_Param[0x25], A
				}
			}
			// Now SMC_Wstate == SMC_PREMARK
		}

		// Mask off this line if DTC interrupts
		SMC_Write_Interrupt();
	}
	else {
		// the new pages are start from beginning of the cluster
		malNpage = SMC_Cluster_Size;

		if (SMC_Cluster_Size != 64){
			if (malNpage > MAL_Block_Numbers) {
				// Do not write the whole cluster
				malNpage = (unsigned char)MAL_Block_Numbers;
				SMC_Wstate |= SMC_POST_ACTION | SMC_WRITE_FLAG;
			}
			else
				SMC_Wstate |= SMC_FULL_ACTION | SMC_WRITE_FLAG;

			SMC_Npage = malNpage;
		}
		else {
			Map_To_DTC_Addr256(malRaddr, SMC_Zone, SMC_Raddr, 0, 0);	// added by HYJ for 2K Flash
			malNpage = malNpage * 4;
			if (malNpage > MAL_Block_Numbers){
				malNpage = (unsigned char)MAL_Block_Numbers;
				SMC_Wstate |= SMC_POST_ACTION | SMC_WRITE_FLAG;
			}
			else
				SMC_Wstate |= SMC_FULL_ACTION | SMC_WRITE_FLAG;

			SMC_Npage512 = malNpage;
			SMC_Npage = malNpage / 4;
		}
		Test_page = 256 - malNpage;
		// Launch DTC to write malPpage pages to malPaddr
		_MAL_BufMgr_Download();		// Switch to DOWNLOAD mode
		DTC_SMC_Write_USB(0);
		// Now	SMC_Wstate = SMC_POSTNEWWRITE or SMC_POSTOLDWRITE
		//		SMC_Wstate = SMC_FULLNEWWRITE or SMC_FULLOLDWRITE
	}
	return MAL_GOOD;
}

#ifdef THUMB_DRIVE
static unsigned short NAND_Blocks;
#pragma DATA_SEG DEFAULT_RAM
unsigned char Startup_VAR = 0;
extern void SMC_Restore_Parameters(void);
extern unsigned char NAND_Format(void);
unsigned char NAND_Init()
{
	unsigned char iChip, MAL_Status;

	DTC_SMC_Init();

	MAL_Status = NAND_Read_Capacity();

	if (MAL_Status != MAL_GOOD)
		return MAL_Status;

/*******************************************
Get the 2K page Flash on board to remap the Bad clusters in the media
*******************************************/
#ifdef Factory_bad
	for (NAND_Chip = 0; NAND_Chip < MAX_NUM_NAND; NAND_Chip++) {
		if (Cluster_Size[NAND_Chip] != 64)
			continue;

		NAND_Select(NAND_Chip);
		SMC_Cluster_Size = 64;
		malPaddr = 0;
		if (!(DTC_SMC_Chk_Spare(format) & 0x02)) {
			unsigned long *paddr;

			malNpage = 0;
			SubBlock = 1;
			DTC_SMC_Mark_Bad();

			malPaddr = 0x4000;
			malNpage = SMC_Cluster_Size;
			Buffer_Param[0x3e] = (Chip_Size[NAND_Chip]/8); //number of zones

			asm {				//initialise DTC buffer
				CLR		A
				CLR		X
	Clr_Buf0:	LD		(Buffer_0, X), A
				LD		(Buffer_0:256, X), A
				DEC		X
				JRNE	Clr_Buf0
			}

			DTC_SMC_Chk_Spare(init);
			paddr = (unsigned long *) Buffer_0;
			while (*paddr & 0x08) {
				malNpage = 0;
				SubBlock = 1;
				malPaddr = *paddr;
				DTC_SMC_Mark_Bad();
				paddr++;
			}

			// Format the chip in very first time of device power on
			asm RIM;
			NAND_Format();
			while (MAL_Block_Numbers);
			asm NOP;
		}
	}
#endif

	if(!Startup_VAR )
		SMC_Restore_Parameters();

	if (Cluster_Size[0] != 64) {
		extern unsigned char Flash_State;
		if ((Flash_State & 0x01) == 0) {
			// The first chip is not a 2K flash and is a new chip
			// Format it and copy in the system blocks
			asm RIM;
			NAND_Format();
			while (MAL_Block_Numbers != 0);
			asm NOP;
		}
	}

	SMC_Lookup_Zone0 = SMC_Lookup_Zone1 = -1;	// There is no zone in both table
	NAND_Lookup_Update0(0, 0);		// Assume chip 0 must exist
	if (SMC_Cluster_Size != 64){
		if (MAL_Capacity >= 32768) {	// Scan the 2nd zone
			if (Chip_Size[0] > 1)
				NAND_Lookup_Update1(0, 1);
			else {
				unsigned char iChip;
				for (iChip = 1; iChip < MAX_NUM_NAND; iChip++)
					if (Chip_Size[iChip]) {
						NAND_Lookup_Update1(iChip, 0);
						break;
					}
			} 
		}
	}
	else {
		if(MAL_Capacity >= 512000) {
			if(Chip_Size[0] >= 16)
				NAND_Lookup_Update1(0, 1);
			else { 
				unsigned char iChip;
				for (iChip = 1; iChip < MAX_NUM_NAND; iChip++)
					if (Chip_Size[iChip]) {
						NAND_Lookup_Update1(iChip, 0);
						break;
					}
			}
		}
	}
	return MAL_GOOD;
}
#else // THUMB_DRIVE
unsigned char SMC_Init()
{
	unsigned char MAL_Status;

	DTC_SMC_Init();

	MAL_Status = SMC_Read_Capacity();

	if (MAL_Status != MAL_GOOD)
		return MAL_Status;

	SMC_Lookup_Zone0 = SMC_Lookup_Zone1 = -1;	// There is no zone in both table
	SMC_Lookup_Update0(0);
	if (MAL_Capacity >= 32768) {
		if(SMC_Cluster_Size != 64)
			// Scan the 2nd zone 
			SMC_Lookup_Update1(1);
		else {
			if(MAL_Capacity == 512000)
				SMC_Lookup_Update1(1);
		}
	}
	return MAL_GOOD;
}
#endif // THUMB_DRIVE

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Variables and functions for SMC formation
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
unsigned char SMC_Format_State;
#define	STEP_CHECK		0x01
#define STEP_PRE_ERASE	0x02
#define STEP_WRITE		0x03
#define STEP_READ		0x04
#define STEP_POST_ERASE	0x05

#define LED_ON_OFF	{ PFDR ^= 0x18; }

#ifdef THUMB_DRIVE

void NAND_Format_After(void)
{
	extern void Logical_Format(unsigned char iCard);

	SMC_Format_State = 0;
	DTC_Current_Func = 0;
	MAL_State = MAL_IDLE;

	SMC_Init_Param();

	NAND_Init();
/*
	if (Chip_Size[0] == 1)
		Logical_Format(2);		// 16MB
	else if (Chip_Size[0] == 2)
		Logical_Format(3);		// 32MB
	else if (Chip_Size[0] == 4)
		Logical_Format(4);		// 64MB
	else if (Chip_Size[0] == 8)
		if (Cluster_Size[0] == 32)
			Logical_Format(5);	// 128MB
		else
			Logical_Format(6);	// 128MB-2K
	else if (Chip_Size[0] == 16)
		Logical_Format(7);		// 256MB-2K
	else if (Chip_Size[0] == 32)
		Logical_Format(8);		// 512MB-2K
*/
	DESELECT_NAND;

//	LED_OFF;	// testing

	return;
}

void NAND_Format_Interrupt(void)
{
Formation_loop:
// Here I need to refresh a timer to ensure that the DTC formatting does not deadlocked

	LED_ON_OFF;

	DTC_Current_Func = DTC_SMC_FORMAT;
	switch (SMC_Format_State) {
	case STEP_CHECK:
	//	if(Buffer_Param[0x26] == 0x02)
	//		asm nop;
		DTC_SMC_Format_Chk(format);
		if (DTC_SMC_Format_Chk(format) & 0x02)
			asm nop;

		SMC_Format_State++;	// going to move to STEP_PRE_ERASE

		DTC_SMC_Format_Erase();
		return;
	case STEP_PRE_ERASE:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		SMC_Format_State++;	// going to move to STEP_WRITE
		DTC_SMC_Format_Write();
		return;
	case STEP_WRITE:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		SMC_Format_State++;	// going to move to STEP_READ
		if (SMC_Cluster_Size == 64)
			Buffer_Param[0x3f] = malNpage;

		DTC_SMC_Format_Read();
		return;
	case STEP_READ:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		if (SMC_Cluster_Size == 64){
			if (Buffer_Param[0x3f] != 0x00){
				DTC_SMC_Format_Read();
				return;
			}
		}
		SMC_Format_State++;	// going to move to STEP_POST_ERASE
		DTC_SMC_Format_Erase();
		return;
	case STEP_POST_ERASE:
		if (DTC_Error & 0x02) {
mark_bad_cluster:
			DTC_SMC_Mark_Bad();
			if(SMC_Cluster_Size == 64)
				Buffer_Param[0x26] = Buffer_Param[0x12];	//as 0xe6 may contain 2 after markpages			
			
			malNpage = SMC_Cluster_Size;	// Pages in a cluster
			break;
		}
	}

format_next_cluster:
	MAL_Block_Numbers--;
	NAND_Blocks--;
	if (NAND_Blocks) {	// move to the next cluster
		if(SMC_Cluster_Size != 64){
			asm {		//		malPaddr += 32
				LD		A, malPaddr:3
				ADD		A, #32
				LD		malPaddr:3, A
				JRNC	lab2
				INC		malPaddr:2
				JRNE	lab2
				JRT		lab1
			}
		}
		else {
			asm {		//		malPaddr += 64 << 8;
				LD		A,malPaddr:2
				ADD		A,#64
				LD		malPaddr:2, A
				JRNC	lab2
lab1:			INC		malPaddr:1
				JRNE	lab2
				INC		malPaddr 
lab2:
			}
		}
	}
	else {
Format_Another_Chip:
		NAND_Chip++;
		if (NAND_Chip == MAX_NUM_NAND) {	// all clusters have been formated
			asm	NOP;
			NAND_Format_After();
			return;
		}

		if (Chip_Size[NAND_Chip] == 0)
			goto Format_Another_Chip;

		NAND_Select(NAND_Chip);
		if (SMC_Cluster_Size != 64)
			asm {
				LD		X, NAND_Chip
				LD		A, (Chip_Size, X)
				SLL		A
				SLL		A
				LD		NAND_Blocks, A
				CLR		A
				LD		NAND_Blocks:1, A
			}
		//	NAND_Blocks = Chip_Size[NAND_Chip] * 1024;
		else 
			asm {
				LD		X, NAND_Chip
				LD		A, (Chip_Size, X)
				SRL		A
				LD		NAND_Blocks, A
				CLR		A
				LD		NAND_Blocks:1, A
			}
		//	NAND_Blocks = Chip_Size[NAND_Chip] * 128;

		malPaddr = 0;
		malNpage = SMC_Cluster_Size; //cluster size could be different
	}

	SMC_Format_State = STEP_CHECK;
	goto Formation_loop;
}

unsigned char NAND_Format(void)
{
	unsigned char iChip, nZone;
	unsigned char TotalSize;

	TotalSize = 0;
	MAL_Block_Numbers = 0;
	for (iChip = 0; iChip < MAX_NUM_NAND; iChip++){
		nZone = NAND_Capacity(iChip);
		TotalSize += nZone;
		if(Cluster_Size[iChip] == 32)
			asm {
				LD		A, nZone
				SLL		A
				SLL		A
				JRT		Add_Block_Num
			}
		//	MAL_Block_Numbers += nZone * 1024;
		else
			asm {
				LD		A, nZone
				SRL		A				// nZone can not be an even number
Add_Block_Num:	ADD		A, MAL_Block_Numbers
				LD		MAL_Block_Numbers, A
			}
		//	MAL_Block_Numbers += nZone * 128;
	}
	
	if (TotalSize == 0 || Chip_Size[0] == 0)
		return MAL_CARD_UNKNOWN;	
	

	// MAL_Capacity is used to report the format progress
	MAL_Capacity = MAL_Block_Numbers;

	// Start from chip0
	NAND_Chip = 0;
	NAND_Select(0);
	SMC_Cluster_Size = Cluster_Size[0];

	if(SMC_Cluster_Size != 64)
		asm {
			LD		A, Chip_Size
			SLL		A
			SLL		A
			LD		NAND_Blocks, A
			CLR		A
			LD		NAND_Blocks:1, A
		}
	//	NAND_Blocks = Chip_Size[0] * 1024;
	else
		asm {
			LD		A, Chip_Size
			SRL		A
			LD		NAND_Blocks, A
			CLR		A
			LD		NAND_Blocks:1, A
		}
	//	NAND_Blocks = Chip_Size[0] * 128;

	malPaddr = 0;
	malNpage = SMC_Cluster_Size;	// Pages in a cluster
	SMC_Format_State = STEP_CHECK;

	// SMC_Format_Interrupt(1);	// I can not start with this,
								// because the compiler will treat the routine as non-interrupt
								// so that the variables will be allocated overlapped
	DTC_Current_Func = DTC_SMC_FORMAT;
	DTC_SMC_Format_Start();		// Use this way to start the procedure, it will generate an interrupt

	return MAL_GOOD;
}

#else	// THUMB_DRIVE

void SMC_Format_Interrupt()
{
Formation_loop:
// Here I need to refresh a timer to ensure that the DTC formatting does not deadlocked

	DTC_Current_Func = DTC_SMC_FORMAT;
	switch (SMC_Format_State) {
	case STEP_CHECK:
		if (DTC_SMC_Format_Chk() & 0x02) {
		//	asm nop;
#ifndef	FORCE_ERASE
//			break;
#endif
		}
		SMC_Format_State++;			// going to move to STEP_PRE_ERASE
		DTC_SMC_Format_Erase();
		return;
	case STEP_PRE_ERASE:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		SMC_Format_State++;			// going to move to STEP_WRITE
		DTC_SMC_Format_Write();
		return;
	case STEP_WRITE:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		SMC_Format_State++;			// going to move to STEP_READ
		if (SMC_Cluster_Size == 64)
		{
			Buffer_Param[0x3f] = malNpage;
		}
		DTC_SMC_Format_Read();
		return;
	case STEP_READ:
		if (DTC_Error & 0x02)
			goto mark_bad_cluster;
		if (SMC_Cluster_Size == 64)
		{
			if (Buffer_Param[0x3f] != 0x00)
			{
				DTC_SMC_Format_Read();
				return;
			}
		}
		SMC_Format_State++;			// going to move to STEP_POST_ERASE
		DTC_SMC_Format_Erase();
		return;
	case STEP_POST_ERASE:
		if (DTC_Error & 0x02) {
mark_bad_cluster:
			DTC_SMC_Mark_Bad();

			malNpage = SMC_Cluster_Size;	// Pages in a cluster
			break;
		}

	}

format_next_cluster:
	MAL_Block_Numbers--;
	if (MAL_Block_Numbers == 0) {		// all clusters have been formated
		SMC_Format_State = 0;
		MAL_State = MAL_IDLE;
		DTC_Current_Func = 0;
		SMC_Init();
		return;
	}

	if (SMC_Cluster_Size != 64)
		malPaddr += SMC_Cluster_Size;	// move to the next cluster
	else
	{
		malPaddr += (SMC_Cluster_Size << 8);
	}

	SMC_Format_State = STEP_CHECK;
	goto Formation_loop;
}

unsigned char SMC_Format()
{
	unsigned char	CardID;

	// Know the card capacity
	CardID = DTC_SMC_ReadID();

	// MAL_Block_Numbers is used as number of blocks in the medium
	SMC_Cluster_Size = 16;
	if (CardID == 0xE3 || CardID == 0xE5)
		MAL_Block_Numbers = 0x200;		// 4MB
	else if (CardID == 0xE6)
		MAL_Block_Numbers = 0x400;		// 8MB
	else {
		SMC_Cluster_Size <<= 1;
		MAL_Block_Numbers = 0x400;
#define hMAL_Block_Numbers *((unsigned char *)&MAL_Block_Numbers)
		switch (CardID) {
		case 0x79:		// 128MB
			hMAL_Block_Numbers <<= 1;
		case 0x76:		// 64MB
			hMAL_Block_Numbers <<= 1;
		case 0x75:		// 32MB
			hMAL_Block_Numbers <<= 1;
		case 0x73:		// 16MB
			break;
		case 0xf1:		// 2k 128mB
			SMC_Cluster_Size <<= 1;
			DTC_SMC_Address_bytes = 4;
			break;
		case 0xda:		//2k 256MB
			SMC_Cluster_Size <<= 1;
			hMAL_Block_Numbers <<= 1;
			DTC_SMC_Address_bytes = 5;
			break;
		default:
			return MAL_CARD_UNKNOWN;
		}
	}

	MAL_Capacity = MAL_Block_Numbers;	// Number of clusters in the media
	if (SMC_Cluster_Size != 64) {
		if (MAL_Block_Numbers <= 2048) {
			DTC_SMC_Address_bytes = 3;		// Less than 32MB		 
		}
		else {
			DTC_SMC_Address_bytes = 4;		// 64MB & 128MB
		}
	}
	// 64MB & 128MB

	malPaddr = 0;
	malNpage = SMC_Cluster_Size;	// Pages in a cluster
	SMC_Format_State = STEP_CHECK;

	// SMC_Format_Interrupt(1);	// I can not start with this,
								// because the compiler will treat the routine as non-interrupt
								// so that the variables will be allocated overlapped
	DTC_Current_Func = DTC_SMC_FORMAT;
	DTC_SMC_Format_Start();		// Use this way to start the procedure, it will generate an interrupt

	return MAL_GOOD;
}
#endif	// #else THUMB_DRIVE

/*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-
	Interrupt from DTC stop
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*/
void SMC_DTC_Int()
{
	unsigned char	ECC;
//#define debug
#ifdef debug
	unsigned char trial, temp;
#endif

	DTC_Error = DTCSTATUS & 0x03;
	
	if (DTC_Current_Func == DTC_SMC_READ) {
		DTC_Current_Func = 0;
		if (SMC_Cluster_Size == 64) {	//for 256MB
			ECC = malNpage - DTC_Page_Counter;
			Buffer_Param[0x00]= 0;
			malNpage = ((malNpage - DTC_Page_Counter)/ 4) ;
			malPaddr += (malNpage << 8);
/*
		if (Buffer_Param[0x01] == 4) {
				malPaddr &= 0xffffff00;
				malPaddr |= 0x00000002;
				Buffer_Param[0x01] = 3;
				Buffer_Param[0x25] = 0x10;
			}
			else if (Buffer_Param[0x01] == 3) {
				malPaddr &= 0xffffff00;
				malPaddr |= 0x00000004;
				Buffer_Param[0x01] = 2;
				Buffer_Param[0x25] = 0x20;
			}
			else if (Buffer_Param[0x01] == 2) {
				malPaddr &= 0xffffff00;
				malPaddr |= 0x00000006;
				Buffer_Param[0x01] = 1;
				Buffer_Param[0x25] = 0x30;
			}
			else {
				malPaddr &= 0xffffff00;
				Buffer_Param[0x01]= 4; //for debug
				Buffer_Param[0x25] = 0x00;
			}
*/
		}
		else
			malPaddr += malNpage - DTC_Page_Counter;
		malNpage = DTC_Page_Counter;
		if (DTC_Error == 0x01) {
			if (DTC_Page_Counter) {
				extern unsigned char SMC_ECC_Correct(char ECC_Buff_Nb);
				//				if (!SMC_ECC_Correct(DTC_Page_Counter))
				if (!SMC_ECC_Correct(ECC)){
					SMC_Read_Interrupt();	// Not correctable ECC error
					return;
				}
				else {						// ECC correction is done
					DTC_Current_Func = DTC_SMC_READ;
					DTC_SMC_Read_USB2();	// Lunch DTC to continue
					return;
				}
			}
			DTC_Error = 0;
			SMC_Read_Interrupt();
		}
		else		// DTC_Error == 2 || DTC_Error == 3
			SMC_Read_Interrupt();
		return;
	}
	else if (DTC_Current_Func == DTC_SMC_WRITE) {
		DTC_Current_Func = 0;
		DTC_Error--;
		if (DTC_Error == 0) {
			if (SMC_Cluster_Size != 64)
				malPaddr += malNpage;		//for 256MB
			else {
#ifdef debug
				//trial = 0; 
				if(MAL_Block_Address == 0x32)
					asm nop;
				trial = malNpage % 4;
#endif			
				malNpage = malNpage / 4;
#ifdef debug
				temp = ((unsigned char *)(&(malPaddr)))[3];
				temp = temp/2;
				trial = trial + temp;
				if (trial >= 4){
					malNpage = malNpage + 1;
					SMC_Npage = SMC_Npage + 1;
				}
#endif
				malPaddr += (malNpage << 8);
				((unsigned char *)(&(malPaddr)))[3] = ((Buffer_Param[0x24] + (unsigned char)0x02) & 0x06);  //new test			
			}	
//			if ((MAL_Block_Numbers != 1) && (i==1))
//				DTC_Error = 1;
			malNpage = 0;
			SMC_Write_Interrupt();
			return;
		}
		// DTC_Error == 2 || DTC_Error == 3
		if (SMC_Cluster_Size != 64)
			malPaddr += 4;	// Page where the error happen
#ifndef PAGE2K_BAD	
		else
			malPaddr += 1024;		// for 256MB
#endif
		malNpage = 4;				// Pages left to read
		SMC_Write_Interrupt();
		return;
	}
	else if (DTC_Current_Func == DTC_SMC_FORMAT) {
		DTC_Current_Func = 0;
#ifdef THUMB_DRIVE
		NAND_Format_Interrupt();
#else
		SMC_Format_Interrupt();
#endif
	}
}

unsigned char SMC_Verify(void)
{
	return MAL_GOOD;
}

unsigned char SMC_Check_Write_Protect()
{
	return 0;
}

void SMC_Break()
{	
	DESELECT();		// Chip deselect
	if (MAL_State != MAL_IDLE) {
		// Stop the DTC execution despite its state
		Stop_DTC();		
		MAL_Finish(FALSE);
		MAL_State = MAL_IDLE;
		DTC_Current_Func = 0;
	}
}

#pragma NO_RETURN
// IMPORTANT: The sequence of this jump table can not be changed
// New line can be appended at the end of the table
void MAL_SMC_Funcs(void)
{
#ifdef THUMB_DRIVE
	asm {
		jp	NAND_Init				// 0
		jp	SMC_Read				// 1
		jp	SMC_Write				// 2
		jp	SMC_Verify				// 3
		jp	NAND_Format				// 4
		jp	NAND_Read_Capacity		// 5
		jp	SMC_Check_Write_Protect	// 6
		jp	SMC_Break				// 7
//		jp	SMC_DTC_Int				// 8
	}
#else
	asm {
		jp	SMC_Init				// 0
		jp	SMC_Read				// 1
		jp	SMC_Write				// 2
		jp	SMC_Verify				// 3
		jp	SMC_Format				// 4
		jp	SMC_Read_Capacity		// 5
		jp	SMC_Check_Write_Protect	// 6
		jp	SMC_Break				// 7
//		jp	SMC_DTC_Int				// 8
	}
#endif
}
